@@ -14,6 +14,7 @@ import concurrent
 import os
 import sys
 import threading
+import time
 from functools import partial
 from typing import Any, Awaitable, Callable, Optional, Tuple, Type, TypeVar, Union
 
@@ -24,10 +25,15 @@ from tornado.concurrent import Future, future_set_exc_info, is_future
 import cognite.geospatial.internal
 from cognite.geospatial._spatial_object import SpatialObject
 from cognite.geospatial.internal import (
+    CoreSpatialItemDTO,
+    CreateSpatialItemsDTO,
     ExternalIdDTO,
     GeometryProjectionDTO,
     InternalIdDTO,
+    RasterGeometrySpatialDTO,
+    SpatialDataDTO,
     SpatialDataRequestDTO,
+    SpatialDataTypeDTO,
     SpatialFilterDTO,
     SpatialIdsDTO,
     SpatialItemsProjectionDTO,
@@ -250,14 +256,17 @@ class CogniteGeospatialClient:
                     if data is not None and len(data.items) > 0:
                         ditem = data.items[0]
                         byte_buffer = base64.urlsafe_b64decode(ditem.value)
-                        if ditem.type == "double":
+                        if ditem.type == SpatialDataTypeDTO.DOUBLE:
                             vector = np.frombuffer(byte_buffer, dtype=">d")
                             spatial_object.add_double(data_item.name, vector)
-                        elif ditem.type == "boolean":
+                        elif ditem.type == SpatialDataTypeDTO.INTEGER:
+                            vector = np.frombuffer(byte_buffer, dtype=">i")
+                            spatial_object.add_integer(data_item.name, vector)
+                        elif ditem.type == SpatialDataTypeDTO.BOOLEAN:
                             vector = np.frombuffer(byte_buffer, dtype=np.uint8)
                             bit_array = np.unpackbits(vector, bitorder="little")
                             spatial_object.add_boolean(data_item.name, np.array(bit_array, dtype=bool))
-                        elif ditem.type == "text":
+                        elif ditem.type == SpatialDataTypeDTO.TEXT:
                             spatial_object.add_text(data_item.name, str(byte_buffer, "utf-8"))
 
         except ApiException as e:
@@ -482,6 +491,194 @@ class CogniteGeospatialClient:
         result = self._run_sync(run_func, self.timeout)
         return result
 
+    @gen.coroutine
+    def create_raster_spatial_async(
+        self,
+        name: str = None,
+        external_id: str = None,
+        description: str = None,
+        metadata=None,
+        source: str = None,
+        crs: str = None,
+        row_min: int = None,
+        row_max: int = None,
+        column_min: int = None,
+        column_max: int = None,
+    ):
+        geometry = RasterGeometrySpatialDTO(
+            row_min=row_min,
+            row_max=row_max,
+            column_min=column_min,
+            column_max=column_max,
+            local_vars_configuration=self.configuration,
+        )
+        spatial_item = CoreSpatialItemDTO(
+            name=name,
+            external_id=external_id,
+            description=description,
+            metadata=metadata,
+            source=source,
+            geometry=geometry,
+            crs=crs,
+        )
+
+        create_spatial_items = CreateSpatialItemsDTO(items=[spatial_item])
+        try:
+            response = yield self.api.create_spatial(self.project, create_spatial_items)
+            if response is not None:
+                return response.items
+            return None
+        except ApiException as e:
+            _throw_exception(e)
+
+    def create_raster_spatial(
+        self,
+        name: str = None,
+        external_id: str = None,
+        description: str = None,
+        metadata=None,
+        source: str = None,
+        crs: str = None,
+        row_min: int = None,
+        row_max: int = None,
+        column_min: int = None,
+        column_max: int = None,
+    ):
+        run_func = partial(
+            self.create_raster_spatial_async,
+            id,
+            name,
+            external_id,
+            description,
+            metadata,
+            source,
+            crs,
+            row_min,
+            row_max,
+            column_min,
+            column_max,
+        )
+        result = self._run_sync(run_func, self.timeout)
+        return result
+
+    @gen.coroutine
+    def add_spatial_item_data_async(self, id: int, name: str, timestamp: int, type, value):
+        value_buff = None
+        if type == SpatialDataTypeDTO.DOUBLE:
+            value_buff = value.astype(">d").tobytes()
+        elif type == SpatialDataTypeDTO.INTEGER:
+            value_buff = value.astype(">i").tobytes()
+        elif type == SpatialDataTypeDTO.BOOLEAN:
+            value_buff = value.astype(">b").tobytes()
+        elif type == SpatialDataTypeDTO.TEXT:
+            value_buff = bytearray(value, encoding="utf-8")
+
+        byte_buffer = base64.urlsafe_b64encode(value_buff)
+
+        spatial_data = SpatialDataDTO(
+            item_id=InternalIdDTO(id=id), name=name, type=type, value=str(byte_buffer, "utf-8"), timestamp=timestamp
+        )
+
+        try:
+            response = yield self.api.add_spatial_item_data(self.project, spatial_data)
+            if response is not None:
+                return response.items
+            return None
+        except ApiException as e:
+            _throw_exception(e)
+
+    @gen.coroutine
+    def calculate_spatial_coverage_async(self, id: int = None, external_id: str = None):
+        """Calculate spatial item coverage by internal ids or external ids.
+        """
+        _check_either_external_id(id, external_id)
+        if id is not None:
+            item = InternalIdDTO(id=id)
+        elif external_id is not None:
+            item = ExternalIdDTO(external_id=external_id)
+        spatial_by_ids = SpatialIdsDTO(items=[item])
+
+        try:
+            response = yield self.api.calculate_spatial_coverage(self.project, spatial_by_ids)
+            return response
+        except ApiException as e:
+            _throw_exception(e)
+
+    def calculate_spatial_coverage(self, id: int = None, external_id: str = None):
+        """Calculate spatial item coverage by internal ids or external ids.
+        """
+        run_func = partial(self.calculate_spatial_coverage_async, id, external_id)
+        result = self._run_sync(run_func, self.timeout)
+        return result
+
+    @gen.coroutine
+    def save_raster_spatial_async(
+        self,
+        name: str = None,
+        external_id: str = None,
+        description: str = None,
+        metadata=None,
+        source: str = None,
+        crs: str = None,
+        data=None,
+    ):
+        item = None
+        if external_id is not None:
+            item = yield self.get_spatial_info_async(external_id=external_id)
+        if item is None:
+            row_min, row_max, column_min, column_max = 0, 0, 0, 0
+
+            try:
+                rows = data["Row"]
+                row_min, row_max = rows.min().astype(np.int64).item(), rows.max().astype(np.int64).item()
+            except KeyError:
+                row_min, row_max = 0, 0
+
+            try:
+                columns = data["Column"]
+                column_min, column_max = columns.min().astype(np.int64).item(), columns.max().astype(np.int64).item()
+            except KeyError:
+                column_min, column_max = 0, 0
+
+            items = yield self.create_raster_spatial_async(
+                name, external_id, description, metadata, source, crs, row_min, row_max, column_min, column_max
+            )
+            if items is not None and len(items) > 0:
+                item = items[0]
+        if item is not None:
+            timestamp = round(time.monotonic() * 1000)
+            for name in data:
+                vector = data[name]
+                spatial_type = None
+                if isinstance(vector, str):
+                    spatial_type = SpatialDataTypeDTO.TEXT
+                elif vector.dtype == "float64" or vector.dtype == "float32":
+                    spatial_type = SpatialDataTypeDTO.DOUBLE
+                elif vector.dtype == "int64" or vector.dtype == "int32":
+                    vector = vector.astype(np.int32)
+                    spatial_type = SpatialDataTypeDTO.INTEGER
+                elif vector.dtype == "bool":
+                    spatial_type = SpatialDataTypeDTO.BOOLEAN
+
+                r = yield self.add_spatial_item_data_async(item.id, name, timestamp, spatial_type, vector)
+
+        yield self.calculate_spatial_coverage_async(id=item.id)
+        return item
+
+    def save_raster_spatial(
+        self,
+        name: str = None,
+        external_id: str = None,
+        description: str = None,
+        metadata=None,
+        source: str = None,
+        crs: str = None,
+        data=None,
+    ):
+        run_func = partial(self.save_raster_spatial_async, name, external_id, description, metadata, source, crs, data)
+        result = self._run_sync(run_func, self.timeout)
+        return result
+
     def _create_geometry(self, id: int = None, external_id: str = None, geography: str = None):
         _check_id_geometry(id, external_id, geography)
         if id is not None:
