@@ -117,7 +117,7 @@ class FileAPI(API):
         """
         file = self.identify(id, name)
         request = SegYQueryRequest(file=file)
-        return [i for i in self.query.GetSegYFile(request, metadata=self.metadata)]
+        return (i for i in self.query.GetSegYFile(request, metadata=self.metadata))
 
     def get_segy_by_lines(
         self,
@@ -147,7 +147,7 @@ class FileAPI(API):
         bottom_right = PositionQuery(iline=bottom_right_inline, xline=bottom_right_crossline)
         rectangle = LineBasedRectangle(top_left=top_left, bottom_right=bottom_right)
         request = SegYQueryRequest(file=file, lines=rectangle)
-        return [i for i in self.query.GetSegYFile(request, metadata=self.metadata)]
+        return (i for i in self.query.GetSegYFile(request, metadata=self.metadata))
 
     def get_segy_by_geometry(
         self, id: Optional[str] = None, name: Optional[str] = None, crs: str = None, wkt: str = None, geo_json=None
@@ -174,7 +174,7 @@ class FileAPI(API):
             geo = Geometry(crs=CRS(crs=crs), wkt=Wkt(geometry=wkt))
 
         request = SegYQueryRequest(file=file, polygon=geo)
-        return [i for i in self.query.GetSegYFile(request, metadata=self.metadata)]
+        return (i for i in self.query.GetSegYFile(request, metadata=self.metadata))
 
     def get_line_range(self, id: Optional[str] = None, name: Optional[str] = None):
         """Get the minimum and maximum crosslines and inlines
